package Client::Alarm;
use Client;
use strict;
use Events;
use Debug;
use Util;

# see GUI.EventLog.info

sub GetAlarm {
  my($q) = @_;
  require "Grid.pm";
  my($type, $comp, $ev, $desc, $severity, $code, $title);

  if ($q->{event}) {
    my $event = State->readEvent($q->{event});
    if (!$event) {
       print Client->error("xml", 110, "Event key not found!");
       return;
    }
    my $H     = State->eventHash($event->[4]);

    $code     = $H->{GridCode};
    $severity = int($event->[0]+0.5);
    $desc     = $event->[1];
    $ev       = Grid->getInfo($type, $comp, {code => $code});
    $title    = $H->{TargetName};
    $comp     = $q->{comp};

print <<EOF;
<?xml version="1.0"?>
EOF

    print "<ALARM>\n";

    print "<CODE>";
    print $code;
    print "</CODE>";
    print "\n";

    print "<SEVERITY>";
    print $severity;
    print "</SEVERITY>";
    print "\n";

    print "<COMPONENT>";
    print $ev->{comp};
    print "</COMPONENT>";
    print "\n";

    print "<DESC>";
    print $desc;
    print "</DESC>";
    print "\n";

    print "<INFO>";
    print $ev->{info};
    print "</INFO>";
    print "\n";
    print "<RECOMMENDED_ACTION>";
    print $ev->{action};
    print "</RECOMMENDED_ACTION>";
    print "\n";

    print "</ALARM>\n";
    print "\n";

  }
}

use vars qw($SEV $TIME1);

sub summary {
  require State;

  if (!$SEV || time - $TIME1 > 60) {
    $SEV = State->sev_summary();
    $TIME1 = time;
  }

  my $date = Util->get_today();
  $SEV->[3]+= 0;
  $SEV->[2]+= 0;
  $SEV->[1]+= 0;

  print "<alarmSummary>\n";
  print " <date>$date</date>\n";
  print " <down>$SEV->[3]</down>\n";
  print " <critical>$SEV->[2]</critical>\n";
  print " <major>$SEV->[1]</major>\n";
  print " <minor>0</minor>\n";
  print "</alarmSummary>";

}

#  Display eventLog information:
#
#  Client::Alarm::eventLog&type=6020&severity=2&key=0x301.5405318.402445&max=1000
#  arguments are optional
#  type : device_type
#  Severity: 1=warning,2=error,3=down,0=informational, default=all
#  key  : device key
#  max  : max number of lines returned, default=1000
#

sub eventLog {
  my($q) = @_;
  
  print Client->http_OK();
  print "<LOG>\n";
  my $F = System->get_home() . "/DATA/Events.log";

  if ($q->{key}) {
    open(O, "/usr/bin/grep \"Enc:$q->{key}\" $F|");
  } elsif ($q->{severity}) {
    open(O, "/usr/bin/grep \"Sev:$q->{severity}\" $F|");
  } else {
    open(O, $F);
  }
  my $max = $q->{max} || 100;
  my($l,$data, $cnt, $date2, $type, $event);
  my($date,$etype,$topic,$desc,$sev,$act,$enc,$mg,$agent,$agg,$ecode);

  while ($l = <O>) {
     chop($l);
     if (substr($l,0,1) eq "L") {
       $data .= substr($l,1) . "\n";
     } else {
       if ($date) {
          &printit($cnt, $date2,$sev,$act,$desc,$ecode,$type,$event,$enc,$topic,$data);
       }
       $data = "";
       ($date,$etype,$topic,$desc,$sev,$act,$enc,$mg,$agent,$agg,$ecode) = split(/\t/, $l);

       next if (substr($date,0,9) eq "RUN-COUNT");
       ($type, $event) = split(/\./, $etype);

       $sev = substr($sev,4);
       $enc = substr($enc,4);
       next if ($q->{severity} && $sev != $q->{severity});
       next if ($q->{type} && $type ne $q->{type});
       next if ($q->{key} && $enc ne $q->{key});
       last if ($cnt++ > $max);
       $date2 = substr($date,0,4) . "-" . substr($date,4,2) . "-" . substr($date,6,2) . " ".
             substr($date,8,2) . ":" . substr($date,10,2) . ":" . substr($date,12,2);
       $act   = substr($act,7);
       $ecode = substr($ecode,6);
     }
  }
  &printit($cnt, $date2,$sev,$act,$desc,$ecode,$type,$event,$enc,$topic,$data);
  close(O);
  print "</LOG>\n";
}

sub printit {
 my($cnt, $d,$sev,$act,$desc,$ecode,$type,$event,$enc,$topic,$data) = @_;

   print <<EOF;
 <EVENT ID=\"$cnt\">
   <DATE>$d</DATE>
   <SEVERITY>$sev</SEVERITY>
   <ACTION>$act</ACTION>
   <DESC>$desc</DESC>
   <GRIDCODE>$ecode</GRIDCODE>
   <TYPE>$type</TYPE>
   <EVENT>$event</EVENT>
   <KEY>$enc</KEY>
   <TOPIC>$topic</TOPIC>
   <DATA>$data</DATA>
 </EVENT>
EOF
}


1;

