package Client::Config;
use Client;
use System;
use strict;


# key = t3:value

sub getDeviceInfo {
  my($q) = @_;
  my $key = $q->{key};

  print Client->http_OK();
  my $comment = Util->deserialize("Comments.txt") || {};
  print "<INFO>$comment->{$key}</INFO>\n";
}


# key  = t3:value
# info = text

sub setDeviceInfo {
  my($q) = @_;

  print Client->http_OK();
  my $comment = Util->deserialize("Comments.txt") || {};

  $q->{info} =~ s/\r//g;
  $q->{info} =~ s/\n /\n/g;
  $q->{info} =~ s/<\//< \//g;
  $comment->{$q->{key}} = $q->{info};

  Util->serialize("Comments.txt", $comment);
  print "<INFO>$q->{key}</INFO>\n";

}

#==================================================
#  ENVIRONMENT (setup, timeouts, providers, frequency etc..)
#==================================================
use vars qw($SKIP);
$SKIP= ",edition,fsa,hostname,html_login,mailer,sdt,se_slave,solution,solution_model,sun_help,time_added,topo_mode,version,GO,";

sub getRenv {
  my($q) = @_;
  my $filter = $q->{filter};
  
  my $renv = System->get_renv();
  print Client->http_OK();
  print "<ENV>\n";
  foreach my $e (sort keys %$renv) {
     next if ($filter && ($filter ne substr($e,0,length($filter))  ));
     next if (substr($e,0,4) eq "GSV_" || index($SKIP, ",$e,") >= 0);
     print "  <VALUE ID=\"$e\">$renv->{$e}</VALUE>\n";
  }
  print "</ENV>\n";
}

sub setRenv {
  my($q) = @_;

  print Client->http_OK();
  print "<ENV>\n";

  my($renv, $devs, $hosts,$notifs) = PDM::ConfigFile->read();
  foreach my $e (keys %$q) {
     next if ($e eq "GO" || substr($e,0,6) eq "COOKIE");
     next if (index($SKIP, ",$e,") >= 0);
     $renv->{$e} = $q->{$e};
     print "<VALUE ID=\"$e\">$q->{$e}</VALUE>\n";
  }
  print "</VALUE>\n";

  PDM::ConfigFile->write($renv, $devs, $hosts, $notifs);
}




#==================================================
#  EMAIL
#==================================================

#
# getEmail
# getEmail&email=ccadieux@central
#
sub getEmail {
  my($q) = @_;

  my $name = $q->{email};

  my($renv, $devs, $hosts,$notifs) = PDM::ConfigFile->read();

  print Client->http_OK();
  print "<NOTIFICATION>\n";

  foreach my $n (@$notifs) {
     next if ($name && $n->{email} ne $name);
     print "  <NOTIF ID=\"$n->{_name}\">\n";
     foreach my $e (sort keys %$n) {
        next if (index(",groups,language,event,apply_filters,", ",$e,") >= 0);
        print "    <VALUE ID=\"$e\">$n->{$e}</VALUE>\n";
     }
     print "  </NOTIF>\n";
  }      
  print "</NOTIFICATION>\n";
  
}


#
# 
# updateEmail&_name=notification1&email=newvalue&device=t3
#

sub updateEmail {
  my($q) = @_;
  my $key = $q->{_name};

  my($renv, $devs, $hosts,$notifs) = PDM::ConfigFile->read();

  print Client->http_OK();
  print "<NOTIFICATION>\n";
  my($UPD);

  my $no = substr($key,12) - 1;
  $notifs->[$no]{_name} = "";

  print "<VALUE ID=\"$key\">updated</VALUE>\n";
  foreach my $el (keys %$q) {
       next if ($el eq "GO" || substr($el,0,6) eq "COOKIE");
       $notifs->[$no]{$el} = $q->{$el};
  }

  PDM::ConfigFile->write($renv, $devs, $hosts, $notifs);

  print "</NOTIFICATION>\n";
}

# 
#  deleteEmail&_name=notification1

sub deleteEmail {
  my($q) = @_;

  my $key = $q->{_name};
  my($renv, $devs, $hosts,$notifs) = PDM::ConfigFile->read();

  print Client->http_OK();
  print "<NOTIFICATION>\n";

  my $no = substr($key,12) - 1;
  $notifs->[$no]{_name} = "";

  print "  <VALUE ID=\"$key\">deleted</VALUE>\n";
  print "</NOTIFICATION>\n";

  PDM::ConfigFile->write($renv, $devs, $hosts, $notifs);
  
  
}


sub addEmail {
  my($q) = @_;

  my($renv, $devs, $hosts,$notifs) = PDM::ConfigFile->read();

  print Client->http_OK();
  print "<NOTIFICATION>\n";

  my $no = $#$notifs +2;

  push(@$notifs, 
               {
                    _name  => "notification$no",
                    active => $q->{active}   || "Y",
             apply_filters => "N",              
                    device => $q->{device}   || "*",
                    email  => $q->{email},
                    event  => "*",
                   no_grid => $q->{no_grid}  || "N",
                  priority => $q->{priority} || "*",
                  script   => $q->{script},
                  skip_agg => exists $q->{skip_agg} ? $q->{skip_agg} : 1,
                  type     => $q->{type} || "Email",
               });

  print "<VALUE ID=\"$no\"></VALUE>\n";

  PDM::ConfigFile->write($renv, $devs, $hosts, $notifs) ;

  print "</NOTIFICATION>\n";
}



#========================================
# MONITORED DEVICES
#========================================
  
sub listDevices {
  my($q) = @_;
  print Client->http_OK();
  my ($renv, $devs, $hosts, $notifs)  = PDM::ConfigFile->read();
  print "<DEVICES>\n";

  foreach my $d (@$devs) {
     print "  <DEVICE ID=\"$d->{_name}\">\n";
     foreach my $el (keys %$d) {
        next if ($el eq "_name");
        print "    <VALUE ID=\"$el\">$d->{$el}</VALUE>\n";
     }
     print "  </DEVICE>\n";
  }  
  print "</DEVICES>\n";
} 
  
#  turn monitoring on/off on a specific device
#  http://hostname:7654/rashttp?GO=Client::Config::monitorDevice&key=t3:serial_no&monitor=Y|N

sub monitorDevice {
  my($q) = @_;

  my $active = uc($q->{monitor} || "Y");

  my ($type, $key) = split(/\:/, $q->{key});

  my ($renv, $devs, $hosts, $notifs)  = PDM::ConfigFile->read();

  foreach my $d (@$devs) {
     if ($d->{key} eq $key) {
        print Client->http_OK();
        $d->{active} = $active;
        PDM::ConfigFile->write($renv, $devs, $hosts, $notifs) ;

        print "<DEVICE ID=\"$d->{_name}\"</VALUE>\n";
        print "  <VALUE ID=\"active\">$active</VALUE>\n";
        print "  <VALUE ID=\"key\">$d->{key}</VALUE>\n";
        print "  <VALUE ID=\"type\">$d->{type}</VALUE>\n";
        print "</DEVICE>\n";
        return;
     }
  }

  print Client->error("xml", 301, "Device $q->{key} not found.");
  
  
}


#-----------------------------------------------------------------
# EMAIL THE STORADE CONFIGURATION
# Client::Config::exportConfig&email_address=email&info=comment
#-----------------------------------------------------------------

sub exportConfig {
  my($q) = @_;
  require GUI::DumpConfig;

  print Client->http_OK();
  print "<CONFIG>\n";

  GUI::DumpConfig::dumpConfig1($q);
  print "\n</CONFIG>\n";
  
}

1;
