package Rasagent;
use Repeat;
use Util;
use Debug;
use strict;

sub startstop_cron {
  my($class, $start_cron, $inspectors) = @_;
 
  my $HOME = System->get_home();
  unlink "$HOME/System/cron_started";
  my $F = "/var/spool/cron/crontabs/root";
  my ($rc,$l);
  
  my $E0 = "0,5,10,15,20,25,30,35,40,45,50,55";
  
  my $COMMENT = "# **** Storage Automated Diagnostic Environment ****\n";
  my $E = $E0 . " * * * * $HOME/bin/rasagent -c >> $HOME/log/cron.log 2>&1\n";
  my $I = "1,6,11,16,21,26,31,36,41,46,51,56 * * * * $HOME/bin/rasinspect -c >> $HOME/log/cron_inspect.log 2>&1\n";
  my $P = "0 3 * * * $HOME/sysbin/patch_scheduler -r >> $HOME/log/cron_patch.log 2>&1\n";
  
  if (open(W, ">/tmp/ras_crons")) {
    if (open(O, $F)) {
       while ($l = <O>) {
         if ($l !~ /$HOME\/bin\/rasagent / && 
	     $l !~ /Storage Automated Diagnostic Environment / && 
             $l !~ /$HOME\/bin\/rasinspect / &&
             $l !~ /sysbin\/patch_scheduler/  ) {
             print W $l;
          }
       }
    }
    if ($start_cron eq "C") {
      print W $COMMENT;
      print W $E;
      print W $I if ($inspectors eq "Y");
      print W $P if (-d "/opt/SUNWstade/sysbin");
    }
    close(O); close(W);
    open(O,"/usr/bin/crontab /tmp/ras_crons|");
    my $l = <O>; close(O);
    if ($l) {
       print "Crons update failed: $l \n";
    } else {
      if ($start_cron eq "C") {
         $rc = "\n  ** cron installed.\n";
      } else {
         $rc = "\n  ** cron entry removed.\n";
      }
    }
  } else {
    $rc = "Error: Cannot open /tmp/ras_crons: $!\n";
  }
  return $rc;
}



sub cron_conflicts {
  my($class, $PIDfile) = @_;

  my($cnt, $l, $created, $PID, $pspid, @a);
  my $SIGTERM = 15;
  my($renv, $devices, $hosts, $notifs) = PDM::ConfigFile->read();
  my $email_address = $notifs->[0]{email} || $renv->{admin_email};
  my $device_cnt = $#$devices +1;
  my $host_cnt   = $#$hosts   +1;

  return 0 if (!-f $PIDfile);  # no pidfile

  open(O,$PIDfile); 
  $PID = <O>; close(O);
  $cnt = 0;
  my $sig = $SIGTERM;
  my $repeat = Repeat->new("/tmp/ras_cron_conflicts", 60*60);

  if (!$PID) {
     $repeat->clear();
     return 0;
  }
  my $LB = Labels->read('rasagent');
  #              devices               slaves            
  my $WAIT = (600 * $device_cnt)  + (60 * ($host_cnt+1)) ;
  $WAIT   += 60*60 if ($renv->{solution} eq "N");  # HBAS

  my $last_command;
  while (1) {
     $cnt++;
     if ($cnt > 3) {
        $last_command = Util->readf(System->get_home() . "/DATA/last_run_command");
        Debug->err('FAILING_TO_KILL',$PID, $pspid . ", last command executed: $last_command");
        return 1;
     }
     $sig = 9 if ($cnt == 3);
     my ($pspid, $children) = Util->findProcesses($PID);

     if ($pspid eq $PID) {
       $created =  (stat($PIDfile))[9];
       return 0 if (!$created);
       if (time - $created > $WAIT) {
	 $last_command = Util->readf(System->get_home() . "/DATA/last_run_command");
         if ($repeat->read()) {
            require Mail;
            my $hn = System->hostname();
	    if($renv->{solution} ne "N"){
	      # This is a solution rack, get se/se2 name
	      foreach my $d (@$devices) {
	         if($d->{class} =~ /storage.se/){
		    $hn = $d->{name};
		    last;
		 }
	      }
	    }

            Mail->mail($email_address, "Storage_ADE", "CRON_CONFLICT on $hn", 
                       $LB->expand(cron_hung => $pspid, $hn, $last_command));
         }
         Debug->err(CRON_CONFLICT => 
	    "killing process ID: $pspid, last command executed was: $last_command");

         kill $sig, $pspid;
         kill $sig, @$children if ($#$children >= 0);
         sleep(1);
       } else {
         #Debug->print('AGENT_RUNNING', $pspid);
         return 1;
       }
     } elsif (!$pspid) {
       unlink $PIDfile;
       Debug->print('PID' => "Remove File");
       return 0;
     } elsif ($pspid ne $PID) {
       Debug->err('WRONG_PROGRAM', $pspid);
       kill $sig, $pspid if ($pspid =~ /\d+/);
     }
  }
  return 0; # OK
}


sub create_pid_file {
  my($class, $LB, $PIDfile) = @_;


  if (open(O,">$PIDfile")) {
    Debug->print(PID => $LB->{create_pid});
    print O $$;
    close(O);
  } else {
    Debug->print(PID =>  $LB->expand('cannot_write_pid', $PIDfile));
    exit;
  }
}

sub read_pid_file {
  my($class, $LB, $PIDfile) = @_;
  my $pid;
  if (open(O,$PIDfile)) {
    $pid = <O>; close(O);
  }
  return $pid;
}

sub randomOrder {
  my($class, $dev1, $dev2) = @_;
  my (@devs, %L);

  foreach my $l (@$dev1) {
     $L{rand(1)} = $l;
  }
  foreach my $l (@$dev2) {
     $L{rand(1)} = $l;
  }
  foreach my $l (sort keys %L) {
     push(@devs, $l);
  }
  return \@devs;
}

# add neighbors to new_list if they are not already in done_list

sub find_neighbors {
  my($class, $Config, $dev) = @_;
  my @new;
  my $l = $class->neighbors($Config, $dev);
  foreach my $d1 (@$l) {
     push(@new, $d1) if ($d1->{active} ne "N");
  }
  return \@new;
}

sub neighbors {
  my($class, $Config, $d) = @_;
  my @neighbors = ();
  my $to = TO->readTopo("MERGE-MASTER");
  return [] if (!$to || !$d);
  my $node = $to->nodeByName("$d->{type}:$d->{key}");
  return [] if (!$node);
  my $elist = $d->{event_list};
  my ($this_port, $pno);

  foreach my $e (@$elist) {
     my($type, $target1, $t_port) = split(/\:/, $e->{Target});
     if ($e->{EventType} =~ /\.LinkEvent/) {
        $this_port = $t_port;
        last;
     } elsif ($e->{Component} =~ /^port.(\d+)$/ || $e->{Component} =~ /^fcPort.(\d+)$/) {
        $this_port = $1;
     }
  }
  my $ports = $node->port();
  my $cnt;
  for ($pno = 0; $pno <= $#$ports; $pno++) {
     next if (defined($this_port) && $pno != $this_port);
     my $p = $ports->[$pno];
     next if (!$p);
     my($target) = $to->nodeByName($p);
     next if (!$target);
     last if ($cnt++ > 2); 
     my $dev;
     my $k = $target->name();
     my($t1, $k1) = split(/\:/, $k);
     if (substr($k,0,5) eq "host:") {
       my $h = $Config->hostByKey($k);
       if ($h) {
         $dev = {type => "host",     name => substr($k,5),
                   ip =>  $h->{ip},  ipno => $h->{ipno}, active => $h->{active},
                };
       }
     } elsif ($k1 ne $d->{key}) {
       $dev = $Config->deviceByKey($k);
     }
     push(@neighbors, $dev) if ($dev);
  }
  return \@neighbors;
}

#  loop == 0: inspector
#  loop == 1: agent loop#
#
sub process_module0 {
  my($class, $static_list, $modname, $LB, $loop, $SINFO) = @_;

  if (!eval "require \"Agent/${modname}.pm\"") {
       Debug->err(LOADING => "$modname: $@ \n");
       return(1, []);
  }
  if ($loop == 0) {
    Debug->print1("\n" . $LB->expand("exe_inspector", $modname) );
  }

  my $f = "Agent::$modname";
  my $agent = $f->new();
  return (1,[]) if (!$agent->can('RUN'));

  my $dev_list = $agent->RUN($static_list, $loop, $SINFO);

  return (0, $dev_list);
}

sub run_inspectors {
  my($class, $LB, $SINFO_DB) = @_;
  my $static_list = [];
  my $SINFO = $SINFO_DB->hash();
  my $sleep;
 
  foreach my $modname ('T3MESSAGE','MESSAGE','DSPMESSAGE','TRAPMESSAGE', 'EVENTS', 'PING') {
      my($next, $broke_list) = Rasagent->process_module0([], $modname, $LB, 0, $SINFO);
      if ($#$broke_list >= 0) {
         my $cnt;
         $sleep = 30 if (index("T3MESSAGE,DSPMESSAGE", $modname) >= 0);
         $SINFO_DB->Lock();
         foreach my $d (@$broke_list) {
            next if (!$d->{key});
            $cnt += $class->inspectorListAdd($d, $SINFO, $static_list);
         }
         $SINFO_DB->UnLock();
         Debug->print2("Inspector $modname found problems..") if ($cnt);
      }
  }
  sleep($sleep) if ($sleep);
  return $static_list;
}

# these devices were found by inspectors or MESSAGE agents and will be added to the list
# of devices to monitor the first time around or after 2 complete round-robin cycle.
#
sub inspectorListAdd {
  my($class, $d, $SINFO, $static_list) = @_;
  my $k = $d->{key};
  my $cnt;
  my $renv = System->get_renv();

  my $s_info = $SINFO->{$k} || {};
  if (!$s_info->{last_inspection} || ($s_info->{scheduled_run_count} > $s_info->{last_inspection})) {
     $s_info->{last_inspection} = $s_info->{scheduled_run_count} + 1;
     $class->listAdd($static_list, [$d]);
     $SINFO->{$k} = $s_info;
     $cnt++;
  } else {
     Debug->print2("Device $d->{name} not added: scheduled-run_count($s_info->{scheduled_run_count}) <= last_inspection($s_info->{last_inspection})");
  }
  return $cnt;
}

sub listAdd {
   my($class, $list, $new, $label) = @_;
   my ($cnt,$done);
   foreach my $el (@$list) {
     $done .= ",$el->{key},";
   }
   foreach my $el (@$new) {
      if (index($done, ",$el->{key},") < 0) {
         Debug->print2("adding $label '$el->{name}/$el->{ipno}' to monitoring list..") if ($label);
         push(@$list, $el);
         $done .= ",$el->{key},";
         $cnt++;
      }
   }
   $cnt++;
}




1;

