package TO::3310;
#<copyright>
# ----------------------------------------------------------
# Sun Proprietary/Confidential Code
# Copyright 2001, Sun Microsystems, Inc. All rights reserved.
# ----------------------------------------------------------
#</copyright>

use base 'TO::STORAGE';
use strict;
sub type { '3310'}
sub leaf { 1 }

sub portLabel {
  my($class, $port) = @_;
  return "port-$port";
}

sub getPortCIMId {
  my($node , $port) = @_;    #  0 / 1
  my $pi = $node->portInfo();
  return $pi->[$port]{PortWWN};
}


sub new {
  my($class, $to, $host, $dev, $exists) = @_;

  my $storages  =  $to->[2];
  my $syns      =  $to->[4];
  my $key       = $dev->{key};
  my $type              = $dev->{type};

  my $obj = $exists || {};
  my $org_name          = $obj->{info}{name};
  $obj->{info}{type}    = $type;
  $obj->{info}{class}   = "storage.$type";
  $obj->{info}{name}    = "$type:$key";
  $obj->{info}{BoxName} = $dev->{name};
  $obj->{info}{host}    = $host;
  $obj->{info}{host_type} = "ib";
  $obj->{info}{VendorID}= "SUN";
  $obj->{info}{LGroup}  = "Sun $type";
  $DB::single= 1;

  require Agent::3310;
  my $rc = Agent::3310->getWWN($dev->{ipno});
  $obj->{info}{disk_map} = $rc->{disk_map};
  $obj->{info}{top_disk} = $rc->{top_disk};

  my $report = $rc->{report};

  Agent::3310->getPortWWNS($dev, $report);

  my ($x, $ports, $cnt);
  if ($dev->{type} eq "3510") {
    for ($x=0; $x <= 20; $x++) {
       next if (!exists $report->{"port.$x.id"});
       my $wwn = $report->{"port.$x.wwn"} ;
       $obj->{portInfo}[$x]{PortWWN}      = $wwn;
       $obj->{portInfo}[$x]{LocalPortWWN} = $wwn;
       $obj->{portInfo}[$x]{Slot}         = substr($wwn,2,1);
       $obj->{portInfo}[$x]{id}           = $report->{"port.$x.id"};
       $syns->{$wwn}  = "$type:$key:$x";
       $obj->{info}{port_cnt} = $x + 1;
    }
  }
  $obj->{info}{disk_cnt} = $report->{"info.total_physical_drives"};
  bless($obj, "TO::$type");
  $syns->{$key} = "$type:$key";
  delete $storages->{$org_name} if ($org_name);
  $storages->{"$type:$key"} = $obj;
  return $obj if ($exists); 

  for ($x=1; $x <= 200; $x++) {
     last if (!exists $report->{"components.disk.$x.model"});
     $obj->{diskInfo}[$x-1] = {
                 status => $report->{"components.disk.$x.status"},
                 };
     $cnt++;
  }
  $obj->{port}= [];
  
  return $obj;
}

sub addFromConfig {
  my($class, $topo, $Config) = @_;

  my $devs = $Config->myDevices();
  my $renv = $Config->renv();

  foreach my $dev (@$devs) {
     next if ($dev->{type} ne $class->type());
     next if (!$dev->isMineToMonitor());
     my $k = $dev->{type} . ":" . $dev->key();
     my $exists = $topo->nodeByName($k);
     if (!$exists) {
        my $wwn = $dev->{wwn} || $dev->{wwn2} || $dev->{wwn3} || $dev->{wwn4};
        $exists = $topo->nodeByKeySkipN($wwn,4);
     }
     $class->new($topo, $renv->{hostname}, $dev, $exists);
  }
}

sub statusFunc {
  my($class, $name, $State, $arg, $node) = @_;

  my $Comp    = State->components($name);
  my $i       = $Comp->value("e");
  my $iIB     = $Comp->value("ib");

  my $enc     = TO->val($i->[0], $iIB->[0]);
  my $encInfo = TO->val2(1,$i, $iIB);

  my ($portInfo, $otherInfo, $port);
  my $other;

  my($disk, $diskInfo, $d, $c, $map, @MAP);
  if ($node->{info}{disk_map}) {
    $map = 1;
    @MAP = split(/,/, $node->{info}{disk_map});
  }

  my (@C, @D, $x);
  foreach my $el (keys %$Comp) {
    if ($el =~ /^channel\.(\d+)/) {
      my $p = $1;
      my $val =$Comp->{$el};
      if ($val->[0] > $C[$p][0]) {
         $C[$p] = [$val->[0], $val->[1]];
      }

    } elsif ($el =~ /^components.disk\.(\d+)/) {
      my $p = $1;
      my $val =$Comp->{$el};
      if ($val->[0] > $D[$p][0]) {
         $D[$p] = [$val->[0], $val->[1]];
      }
    }
    my $val =$Comp->{$el};
    $Comp->summ($el, \$other, \$otherInfo);
  }
  shift(@C);  # first channel(port) is 1
  foreach my $e (@C) {
    $port .= "$e->[0],";
    $portInfo .= "$e->[1],";
  }
  my $rows = int($node->{info}{top_disk} / 16);
  my ($r, $d);
  for ($r=0; $r <= $rows; $r++) {
     for ($d=0; $d <= 11; $d++) {
        my $ix = $r*16 + $d;
        my $e = $D[$ix];
        if ($MAP[$ix]) {
          $disk .= "$e->[0],";
          $diskInfo .= "[Encl-$r disk-$d] $e->[1],";
        } else {
          $disk .= "-1,";
          $diskInfo .= "[Encl-$r disk-$d],";
        }

     }
  }

  return ($enc, $port, $disk, $other, $encInfo, $portInfo, $diskInfo, $otherInfo);
}




sub iconLayout {
  my($obj) = @_;
  my ($cat, $label2, $ses);
  my $rows = int($obj->{info}{top_disk} / 16);
  my $cnt = 12;

  my ($x, $dd);
  for ($x=0; $x <= $rows; $x++) {
    $dd .= "$x/";
  }
  chop($dd) if ($dd);
  my $ports = $obj->{info}{port_cnt} || 1;

  $cat = "a,3310,$ports/  ,$cnt,$dd,_,,E0D0F0/3310.gif";

  $label2 = $obj->{info}{LGroup};
  $ses = 1;

  return ($cat, $label2, $ses);
}

# "a,$t3,2/12,-$dt,$dd,LMPVA,Loop/Midplane/Power/Volumes/Alerts,D0D0F0/t3pg.gif";

1;
